@@ -1,5 +1,5 @@
 import os
-from typing import List, Optional   # noqa: F401
+from typing import List, Dict, Optional  # noqa: F401
 
 # PROXY configuration keys
 PROXY_HOST = 'PROXY_HOST'
@@ -39,6 +39,9 @@ class Config:
     # The relationalAttribute name of Atlas Entity that identifies the database entity.
     ATLAS_DB_ATTRIBUTE = 'db'
 
+    # Configurable dictionary to influence format of column statistics displayed in UI
+    STATISTICS_FORMAT_SPEC: Dict[str, Dict] = {}
+
     # whitelist badges
     WHITELIST_BADGES: List[str] = []
 
